# The code below can detect the presence of intermediary hosts 
# in the incoming data from a peer. The code monitors mainly
# the IPT values which are higher than the RTT condition. 

@load mt

type ftpdata_session_info: record {
	id: count;		
	connection_id: conn_id;
	rtt: interval;
	rtt_cond: interval;	
	last_pkt_t: time;
	pkt_t: time;	
	count_p: count;
	sum: interval;
	avg_t: interval;		
	alarmp: bool;			
	};

global ftpdata_sessions: table[conn_id] of ftpdata_session_info;
global ftpdata_session_id = 0;
const proxy_file = open_log_file("proxy") &redef;

function new_ftpdata_session(c: connection)
	{
	local session = c$id;
	local new_id = ++ftpdata_session_id;
	local info: ftpdata_session_info;
	info$id = new_id;
	info$connection_id = session;
	info$rtt = 0sec;
	info$rtt_cond = 0sec;	
	info$last_pkt_t = c$start_time;
	info$pkt_t = c$start_time;	
	info$count_p = 0;
	info$sum = 0 sec;
	info$avg_t = 0 sec;	
	info$alarmp = F;	

	ftpdata_sessions[session] = info;
	}

function is_ftpdata_conn(c: connection): bool
	{
	return c$id$orig_p == 20/tcp ;
	}

event new_packet (c: connection, p: pkt_hdr)
	{

	local id = c$id;
	if ( id !in ftpdata_sessions )
		return;
	local session = ftpdata_sessions[id];
	if (p$ip$src == id$resp_h && p$tcp$flags == 24)
		{
		session$pkt_t = network_time();	
		session$rtt = connection_rtt(c$id);
		if (session$rtt > .05sec)
			session$rtt_cond =  session$rtt*1.25;
		else
			session$rtt_cond =  session$rtt*2.5;
		
		if((session$pkt_t - session$last_pkt_t) > session$rtt_cond )
			{
			session$sum = session$sum + (session$pkt_t - session$last_pkt_t);
			++session$count_p;		
			if(session$count_p > 5 && ! session$alarmp)
				{
				session$avg_t = (session$sum/(session$count_p));
				if(session$avg_t > session$rtt_cond) 
					{
					print proxy_file, fmt ("%.6f: #%s Proxy Connection %s", network_time(), prefixed_id(session$id), id_string(c$id));				
					session$alarmp = T;
					}					
				}	
			}							
		}
	session$last_pkt_t = network_time();	
	}	

event connection_established(c: connection)
	{
	if ( is_ftpdata_conn(c) )
		{		
		local id = c$id;
		if ( id !in ftpdata_sessions )
			new_ftpdata_session(c);			
		}	
	}
