# The given code can detect the presence of a proxy system, 
# in the inital key exchange phase. This will detect the 
# presence of only generic TCP proxy.

@load mt

type ssh_session_info: record {
	id: count;		
	connection_id: conn_id;
	rtt: interval;
	rtt_cond: interval;	
	last_pkt_t: time;
	pkt_t: time;
	pkt_cnt: count;
	count_p: count;
	sum: interval;
	avg_t: interval;		
	alarmp: bool;			
	};

global ssh_sessions: table[conn_id] of ssh_session_info;
global ssh_session_id = 0;
const proxy_file = open_log_file("proxy") &redef;

function new_ssh_session(c: connection)
	{
	local session = c$id;
	local new_id = ++ssh_session_id;
	local info: ssh_session_info;
	info$id = new_id;
	info$connection_id = session;
	info$rtt = 0sec;
	info$rtt_cond = 0sec;	
	info$last_pkt_t = c$start_time;
	info$pkt_t = c$start_time;	
	info$pkt_cnt = 0;
	info$count_p = 0;
	info$sum = 0 sec;
	info$avg_t = 0 sec;	
	info$alarmp = F;
	
	ssh_sessions[session] = info;
	}

function is_ssh_conn(c: connection): bool
	{
	return c$id$resp_p == 22/tcp ;
	}

event new_packet (c: connection, p: pkt_hdr)
	{
	local id = c$id;
	if ( id !in ssh_sessions )
		return;
	local session = ssh_sessions[id];
	session$rtt = connection_rtt(c$id);
	if (session$rtt > .05sec)
		session$rtt_cond =  session$rtt*1.25;
	else
		session$rtt_cond =  session$rtt*2.5;
	if (p$ip$src == id$orig_h && p$tcp$flags != TH_ACK && session$pkt_cnt < 7)
		{
		session$pkt_t = network_time();	
		++session$pkt_cnt;		
		if((session$pkt_t - session$last_pkt_t) > session$rtt_cond )
			{
			session$sum = session$sum + (session$pkt_t - session$last_pkt_t);
			++session$count_p;		
			if(session$pkt_cnt >= 7 && ! session$alarmp)
				{
				session$avg_t = (session$sum/(session$count_p));
				if(session$avg_t > session$rtt_cond) 
					{
					print proxy_file, fmt ("%.6f: #%s Proxy Connection %s", network_time(), prefixed_id(session$id), id_string(c$id));				
					session$alarmp = T;
					}					
				}	
			}							
		}
	session$last_pkt_t = network_time();	
	}

event connection_established(c: connection)
	{
	if ( is_ssh_conn(c) )
		{		
		local id = c$id;
		if ( id !in ssh_sessions )
			new_ssh_session(c);			
		}	
	}
