@load mt


type telnet_session_info: record {
		id: count;
		state: bool;
		connection_id: conn_id;
		rtt: interval;
		rtt_cond: interval;
		first_pkt_t: time;
		last_pkt_t: time;
		pkt_t: time;
		
		chkcond_f: bool;
		count_p: count;
		sum: interval;
		avg_t: interval;
		
		alarmp: bool;
		pkt_cnt: count;
		nwpchk_f: bool;
		proxycond_f: bool;
	};


global telnet_sessions: table[conn_id] of telnet_session_info;
const rtt_threshold = 1.2 sec;
global first_pkt_t: time;
global type_speed = .25sec;
global type_brk = .7sec;
global telnet_session_id = 0;
global arrival_t : table[conn_id, count] of interval;
const proxy_file = open_log_file("proxy") &redef;


function new_telnet_session(c: connection)
	{
	local session = c$id;
	local new_id = ++telnet_session_id;

	local info: telnet_session_info;
	info$id = new_id;
	info$state = F;
	info$connection_id = session;
	info$rtt = 0sec;
	info$rtt_cond = 0sec;
	info$first_pkt_t = c$start_time;
	info$last_pkt_t = c$start_time;
	info$pkt_t = c$start_time;

	info$alarmp = F;
	info$chkcond_f = F;
	info$count_p = 0;
	info$sum = 0 sec;
	info$avg_t = 0 sec;
	info$pkt_cnt = 1;
	info$nwpchk_f = F;
	info$proxycond_f = T;

	telnet_sessions[session] = info;
	}

function is_telnet_conn(c: connection): bool
	{
	return c$id$resp_p == telnet || c$id$resp_p == rlogin;
	}

event login_success(c: connection, user: string, client_user: string,
			password: string, line: string)
	{
	local id = c$id;
	local session = telnet_sessions[id];		
	session$state= T;	
	}

event new_packet (c: connection, p: pkt_hdr)
	{

	local id = c$id;
	if ( id !in telnet_sessions )
		return;

	local session = telnet_sessions[id];

	if(!session$state)
	{
	session$rtt = connection_rtt(c$id);
	if (session$rtt > .05sec)
		session$rtt_cond = (session$rtt*1.25);
	else
		session$rtt_cond = (session$rtt*2);
	type_brk = type_brk + session$rtt_cond;
	
	if (p$ip$src == id$orig_h && p$tcp$flags != TH_ACK)
		{
		if(session$count_p < 4 && !session$nwpchk_f)
			{
			session$pkt_t = network_time();	
			session$sum = session$sum + (session$pkt_t - session$last_pkt_t);
			++session$count_p;
			++session$pkt_cnt;			
		}
		if(session$count_p >= 4 && ! session$nwpchk_f)
			{
			session$avg_t = (session$sum/(session$count_p));			
			if(session$avg_t > session$rtt_cond) 
				{
				print proxy_file, fmt("%.6f: #%s Proxy Connection using generic TCP proxy %s", network_time(), prefixed_id(session$id), id_string(c$id));				
				session$alarmp = T;
				}			
			session$nwpchk_f = T;
			}
		if(session$nwpchk_f)
			{
			session$sum = 0 sec;
			session$count_p = 0;
			session$pkt_cnt = 0;
			session$avg_t = 0 sec;
			session$pkt_t = c$start_time;
			session$last_pkt_t = c$start_time;
			}
		}
		session$last_pkt_t = network_time();					
		return;	
	}

	if (p$ip$src == id$orig_h && p$tcp$flags != TH_ACK)
		{
			session$pkt_t = network_time();
			if( ((session$pkt_t - session$first_pkt_t) < 90sec && session$pkt_cnt < 10) && ! (p$tcp$flags == TH_ACK + TH_FIN) )
			{
				if((session$pkt_t - session$last_pkt_t) > rtt_threshold)
					session$last_pkt_t = network_time();
								
				if((session$pkt_t - session$last_pkt_t) > 0sec && (session$pkt_t - session$last_pkt_t) < type_brk)
				{
					session$sum = session$sum + (session$pkt_t - session$last_pkt_t);				
					++session$count_p;
					++session$pkt_cnt;
				}				
				session$chkcond_f = T;					
			}	
		
			if ( (!session$chkcond_f || p$tcp$flags == TH_ACK + TH_FIN) && session$proxycond_f)
			{			
			session$avg_t = (session$sum/(session$count_p - 1));
			session$rtt_cond = session$rtt_cond + type_speed;			
			if(session$avg_t > session$rtt_cond && ! session$alarmp) 
				{
					print proxy_file, fmt ("%.6f: #%s Proxy Connection %s", network_time(), prefixed_id(session$id), id_string(c$id));					
				}
			session$proxycond_f = F;			
			}				
			session$last_pkt_t = network_time();
			session$chkcond_f = F;
		}		
	}

event connection_established(c: connection)
	{
	if ( is_telnet_conn(c) )
		{
		local id = c$id;
		if ( id !in telnet_sessions )
			new_telnet_session(c);			
		}	
	}
