# The given code can detect the Tor usage in an incoming HTTP
# request at the web server. The IPT value of first four GET 
# requests are analyzed to check the presence of proxy system.

@load mt

# HTTP Session record to store the IPT values, Id, seesion info.
type http_session_info: record {
	id: count;		
	connection_id: conn_id;
	rtt: interval;
	rtt_cond: interval;	
	last_pkt_t: time;
	pkt_t: time;
	};
# HTTP GET request record to store the GET request IPT value, client add, no. of GET request.
type get_request: record {
	get_time: interval;
	client_addr: addr;
	get_cnt: count;
	get_avg: interval;
	alarmp: bool;	
	};

global http_get_requests: table[addr] of get_request;
global http_sessions: table[conn_id] of http_session_info;
global http_session_id = 0;
const proxy_file = open_log_file("proxy") &redef;
# Function to initialize the new HTTP session
function new_http_session(c: connection)
	{
	local session = c$id;
	local new_id = ++http_session_id;
	local info: http_session_info;
	info$id = new_id;
	info$connection_id = session;
	info$rtt = 0sec;
	info$rtt_cond = 0sec;	
	info$last_pkt_t = c$start_time;
	info$pkt_t = c$start_time;

	http_sessions[session] = info;
	}

function is_http_conn(c: connection): bool
	{
	return c$id$resp_p == 80/tcp ;
	}
# Function to initialize the new GET request
function new_http_get(c: connection)
	{
	local client_a = c$id$orig_h;
	local req: get_request;

	req$get_time = 0 sec;
	req$client_addr = client_a;
	req$get_cnt = 1;
	req$get_avg = 0 sec;
	req$alarmp = F;	

	http_get_requests[client_a] = req;
	}

event new_packet (c: connection, p: pkt_hdr)
	{

	local id = c$id;
	if ( id !in http_sessions )
		return;
	local session = http_sessions[id];
	if(p$ip$src == id$orig_h && p$tcp$flags == 16)
		session$last_pkt_t = network_time();
	if (p$ip$src == id$orig_h && p$tcp$flags == 24)
		{
		session$pkt_t = network_time();	
		if (p$ip$src !in http_get_requests)
			new_http_get(c);
		local get_requestl = http_get_requests[p$ip$src];
		get_requestl$get_time = get_requestl$get_time + (session$pkt_t - session$last_pkt_t);
		++get_requestl$get_cnt;
		if(get_requestl$get_cnt > 4 && !get_requestl$alarmp)
			{
			get_requestl$get_avg = get_requestl$get_time/get_requestl$get_cnt;
			session$rtt = connection_rtt(c$id);
			if (session$rtt > .05sec)
				session$rtt_cond =  session$rtt*1.25;
			else
				session$rtt_cond =  session$rtt*2.5;
			if (get_requestl$get_avg > session$rtt_cond)
				print proxy_file, fmt ("%.6f: #%s Proxy Connection %s", network_time(), prefixed_id(session$id), id_string(c$id));				
			get_requestl$alarmp = T;			
			}

		}		
	}	

event connection_established(c: connection)
	{
	if ( is_http_conn(c) )
		{		
		local id = c$id;
		if ( id !in http_sessions )
			new_http_session(c);							
		}	
	}
